/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"
#include "time.h"
#include "input.h"
#include "display/display.h"
#include "rpg/world/maps/testmap.h"
#include "ui/textbox.h"

#include <font8x8_basic.h>

map_t MAP;
game_t GAME;

void gameInit() {
  memset(&GAME, 0, sizeof(game_t));

  timeInit();
  inputInit();
  displayInit();
  textboxInit();
  
  testMapInit(&MAP);
  gameSetMap(&MAP);
}

uint8_t gameUpdate(const float_t delta) {
  timeUpdate(delta);
  inputUpdate();

  if(GAME.currentMap) {
    mapUpdate(GAME.currentMap);
  }

  displayUpdate();

  if(inputIsDown(INPUT_BIND_CANCEL)) return GAME_UPDATE_RESULT_EXIT;
  return GAME_UPDATE_RESULT_CONTINUE;
}

void gameSetMap(map_t *map) {
  GAME.currentMap = map;
}

void gameDispose() {
  displayDispose();
}