/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "rpg/world/map.h"

#define GAME_UPDATE_RESULT_CONTINUE 0
#define GAME_UPDATE_RESULT_EXIT 1

typedef struct {
  map_t *currentMap;
} game_t;

extern game_t GAME;

/**
 * Initializes the game state.
 */
void gameInit();

/**
 * Updates the game state.
 * 
 * @param delta Time since last update.
 * @return Game update result, 0 for continue, 1 for exit, else for failure.
 */
uint8_t gameUpdate(const float_t delta);

/**
 * Sets the current map, does not take ownership.
 * 
 * @param map Map to set.
 */
void gameSetMap(map_t *map);

/**
 * Cleans up the game state.
 */
void gameDispose();