/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "player.h"

#define ENTITY_TYPE_NULL 0
#define ENTITY_TYPE_PLAYER 1
#define ENTITY_TYPE_NPC 2

#define ENTITY_STATE_IDLE 0
#define ENTITY_STATE_WALKING 1
#define ENTITY_STATE_TALKING 2

typedef struct _map_t map_t;

typedef struct {
  float_t time;
} entitywalkstate_t;

typedef struct _entity_t {
  map_t *map;
  uint8_t type;
  uint16_t x;
  uint16_t y;
  uint8_t layer;
  uint8_t direction;
  uint8_t state;

  // State
  union {
    entitywalkstate_t walk;
  };

  // Type
  union {
    player_t player;
  };
} entity_t;

/**
 * Initializes an entity.
 * 
 * @param entity Entity to initialize.
 * @param type Type of entity to initialize.
 */
void entityInit(
  entity_t *entity,
  const uint8_t type,
  map_t *map
);

/**
 * Updates an entity.
 * 
 * @param entity Entity to update.
 */
void entityUpdate(entity_t *entity);

/**
 * Moves an entity.
 * 
 * @param entity Entity to move.
 * @param dir Direction to move.
 */
void entityWalk(entity_t *entity, const uint8_t dir);

/**
 * Sets the position of an entity. Performs an immediate move.
 * 
 * @param entity Entity to set position of.
 * @param x X position to set.
 * @param y Y position to set.
 */
void entityPositionSet(entity_t *entity, const uint16_t x, const uint16_t y);