/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "linuxhost.h"
#include "game.h"
#include <unistd.h>
#include <termios.h>

int32_t LINUX_TERM_HOST_KEY_PRESS;
struct termios origionalTermios;

void terminalModeReset() {
  tcsetattr(0, TCSANOW, &origionalTermios);
}

void terminalModeSetCONIO() {
  struct termios new_termios;

  /* take two copies - one for now, one for later */
  tcgetattr(0, &origionalTermios);
  memcpy(&new_termios, &origionalTermios, sizeof(new_termios));

  /* register cleanup handler, and set the new terminal mode */
  atexit(terminalModeReset);
  cfmakeraw(&new_termios);
  tcsetattr(0, TCSANOW, &new_termios);
}

int32_t keyboardCharacterAvailable() {
  struct timeval tv = { 0L, 0L };
  fd_set fds;
  FD_ZERO(&fds);
  FD_SET(0, &fds);
  return select(1, &fds, NULL, NULL, &tv) > 0;
}

int32_t keyboardCharacterRead() {
  int32_t r;
  uchar_t c;
  if ((r = read(0, &c, sizeof(c))) < 0) {
    return r;
  } else {
    return c;
  }
}

int32_t linuxHostBeginUpdating() {
  while(true) {
    terminalModeSetCONIO();

    while(!keyboardCharacterAvailable()) {
      uint8_t result = gameUpdate(0.1f);
      if(result == GAME_UPDATE_RESULT_EXIT) return 0;
      if(result != GAME_UPDATE_RESULT_CONTINUE) return -1;

      // Clear input.
      LINUX_TERM_HOST_KEY_PRESS = -1;

      // Clear console and return to left top
      printf("\033[2J\033[1;1H");
      usleep(100000);
    }

    LINUX_TERM_HOST_KEY_PRESS = keyboardCharacterRead();
  }

  return 0;
}